/*********************************************
   EVENTS.C   Unit fr die Ereignisverwaltung
**********************************************/

#include "StdTypes.h"
#include "startup.h"
#include "events.h"


/****** gepufferte Event-Verwaltung ******/

#define eventbuffersize 64             // immer Zweierpotenz
dword EventBuffer[eventbuffersize];
word  evWZ, evRZ;

/****** ladbare Timer ********************/ 
/* d.h. Down-Counters, 
   die bei Erreichen von 0 einen Event
   in den Eventbuffer einspeisen  
*/
DTTE CurrTimers[numtimers];


/****** Funktionen ***********************/
void Event_Init (void)
{ int i;
  evWZ = evRZ = 0;
  i = 0;
  while (i<numtimers)
  { CurrTimers[i].aEvent = 0;
    ++i;
  }
}

bool Event_avail (void)
{ return (evWZ!=evRZ);
}

bool Exists_Event (dword aEvent)
{ word a, e;
  a = evRZ & (eventbuffersize-1);
  e = evWZ & (eventbuffersize-1);
  while (a != e)
  { if (EventBuffer[evRZ] == aEvent) return true;
    a = (a+1) & (eventbuffersize-1);
  }
  return false;
}

void Kill_Event (dword aEvent)
{ word a, e;
  a = evRZ & (eventbuffersize-1);
  e = evWZ & (eventbuffersize-1);
  while (a != e)
  { if (EventBuffer[evRZ] == aEvent) EventBuffer[evRZ] = 0;
    a = (a+1) & (eventbuffersize-1);
  }
  return;
}

dword Get_Event (void)
{ dword w;
  if (evWZ==evRZ) return 0;
  w = EventBuffer[evRZ];
  evRZ = (evRZ+1) & (eventbuffersize-1);
  return w;
}

void Add_Event (dword aEvent)
{ word w;
  w = (evWZ+1) & (eventbuffersize-1);
  if (w==evRZ) return;
  EventBuffer[evWZ & (eventbuffersize-1)] = aEvent;
  evWZ = w;
}


bool Add_Delayed_Event (dword millisec, dword aEvent)
{ int i;
  i = 0;
  while (i<numtimers)
  { if (CurrTimers[i].aEvent == 0)
    { CurrTimers[i].endzeit = Ticks + millisec; /* zuerst die Endzeit */
      CurrTimers[i].aEvent  = aEvent;           /* dann scharf machen */
      return true;
    }
    ++i;
  }
  return false;
}

// lscht einen verzgerten Event aus der TimerTick-Liste und
// lscht ihn vorsorglich auch aus der Event-Liste, fall dieser
// zwischenzeitlich vom TimerTick dorthin befrdert ist
void Kill_Delayed_Event (dword aEvent)
{ int i;
  i = 0;
  while (i<numtimers)
  { if (CurrTimers[i].aEvent == aEvent)
    CurrTimers[i].aEvent = 0;
    ++i;
  }
  Kill_Event(aEvent);
}

bool Exists_Delayed_Event(dword aEvent)
{ int i;
  i = 0;
  while (i<numtimers)
  { if (CurrTimers[i].aEvent == aEvent) return true;
    ++i;
  }
  return false;
}
